==> file4life-1.1.0-source.tgz: file4life-1.1.0/src/F4L/V0/Op.hs <==
{- This file is part of file4life, a file backup program written in Haskell.
 - file4life is Copyright (C) Antonio Bellezza 2014
 - file4life is free software: you can redistribute it and/or modify
 - it under the terms of the GNU General Public License as published by
 - the Free Software Foundation, either version 3 of the License, or
 - (at your option) any later version.

 - file4life is distributed in the hope that it will be useful,
 - but WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 - GNU General Public License for more details.

 - You should have received a copy of the GNU General Public License
 - along with file4life.  If not, see <http://www.gnu.org/licenses/>.
 -}
{-# LANGUAGE MultiParamTypeClasses, FlexibleContexts, RankNTypes #-}

module F4L.V0.Op (
  Op(..),
  Requirement(..)
  ) where

import F4L

import Control.DeepSeq
import Control.Monad.State.Strict
import qualified Data.Binary as B
import qualified Data.ByteString.Short as SBS
import Data.Binary( Binary )
import qualified Data.Map.Strict as M
import Data.Word( Word8 )


data Requirement = MergePriority !Priority
                 | OverridePriority !Priority
                 deriving (Eq, Show)


instance NFData Requirement


instance Binary Requirement where
  put ( MergePriority a ) = B.put a
  put ( OverridePriority a ) = B.put '!' >> B.put a
  -- TODO: avoid code duplication with above
  get = do
    t <- B.get
    case t of
      '!' -> liftM OverridePriority B.get
      '0' -> return $ MergePriority Skip
      'a' -> return $ MergePriority DefaultPriority
      'p' -> liftM ( MergePriority . Priority ) B.get
      'z' -> return $ MergePriority AsManyAsPossible
        

data Op = StartSession SessionVersion SessionId EnvironmentId TimeStamp Int
        | EndSession
        | AddEnvironmentLabel EnvironmentId String
        | AddStorageLabel StorageId String
        | WithStorage StorageId FilePath
        | WithRequire Requirement -- ^ set requirement for following files
        | MTag String -- ^ add tag for following files
        | RetailPath Int FilePath -- ^ remove n chars and append new part
        | PatchPath Int Int FilePath -- ^ cut from pos1 to pos2 and replace with new part
        | BackupPath -- ^ path within storage is standard backup path
        | WithFid FileId
        | WithKnownFid SBS.ShortByteString
        | WithTag String -- ^ add tag for next file or path
        | WithProperty String String -- ^ add property for next file or path
        | Seen -- ^ mark path as seen
        | Mark -- ^ add attributes
        | SeenMark -- ^ Seen + Mark
        | Missing -- ^ no file present in path
        deriving (Show, Eq)


instance NFData Op where
  rnf ( StartSession _ a b _ _ ) = rnf a `seq` rnf b
  rnf ( AddEnvironmentLabel a b ) = rnf a `seq` rnf b
  rnf ( AddStorageLabel a b ) = rnf a `seq` rnf b
  rnf ( WithStorage a b ) = rnf a `seq` rnf b
  rnf ( WithRequire a ) = rnf a
  rnf ( MTag a ) = rnf a
  rnf ( RetailPath a b ) = a `seq` rnf b
  rnf ( PatchPath a b c ) = a `seq` b `seq` rnf c
  rnf ( WithFid a ) = rnf a
  rnf ( WithKnownFid a ) = rnf a
  rnf ( WithTag a ) = rnf a
  rnf ( WithProperty a b ) = rnf a `seq` rnf b
  rnf x = x `seq` ()


{---------------------------------------- Binary ----------------------------------------}

swap (a, b) = (b, a)

freqTags = M.fromList $
           [ ( 'p', "picture" )
           , ( 'm', "email" )
           ]

freqTags' = M.fromList $ map swap $ M.toList freqTags

freqProps = M.fromList $
           [ ( 'D', "exif:DateTime" )
           ]
freqProps' = M.fromList $ map swap $ M.toList freqProps


instance Binary Op where
  put ( StartSession sver sessid env ts l ) = B.put 'S' >> B.put sver >> B.put sessid >> B.put env >> B.put ts >> B.put l
  put EndSession = B.put 'E'
  put ( AddEnvironmentLabel env tag ) = B.put 'L' >> B.put env >> B.put tag
  put ( AddStorageLabel env tag ) = B.put 'l' >> B.put env >> B.put tag
  put ( WithStorage sid path ) = B.put 's' >> B.put sid >> B.put path
  put ( WithRequire req ) = B.put 'r' >> B.put req
  put ( MTag s ) = B.put 'T' >> B.put s
  -- put ( MProperty k v ) = B.put 'P' >> B.put k >> B.put v
  put BackupPath = B.put 'b'
  put ( RetailPath n path ) = B.put 'p' >> B.put ( fromIntegral n :: Word8 ) >> B.put path
  put ( PatchPath d k path ) = B.put 'P' >> B.put ( fromIntegral d :: Word8 ) >> B.put ( fromIntegral k :: Word8 ) >> B.put path
  put ( WithFid fid ) = B.put 'i' >> B.put fid
  put ( WithKnownFid h ) = B.put 'I' >> B.put ( SBS.unpack h )
  put ( WithTag s ) = case M.lookup s freqTags' of
    Nothing ->
      B.put 't' >> B.put s
    Just x ->
      B.put '_' >> B.put x
  put ( WithProperty k v ) = case M.lookup k freqProps' of 
    Nothing ->
      B.put 'a' >> B.put k >> B.put v
    Just x ->
      B.put '^' >> B.put x >> B.put v
  put Seen = B.put '1'
  put Mark = B.put 'm'
  put SeenMark = B.put 'M'
  put Missing = B.put '0'
  -- put Nop = return ()

  get = do
    t <- B.get
    case t of
      'S' -> liftM5 StartSession B.get B.get B.get B.get B.get
      'E' -> return EndSession
      'L' -> liftM2 AddEnvironmentLabel B.get B.get
      'l' -> liftM2 AddStorageLabel B.get B.get
      's' -> liftM2 WithStorage B.get B.get
      'r' -> liftM WithRequire B.get
      'T' -> liftM MTag B.get
      'b' -> return BackupPath
      'p' -> liftM2 RetailPath (liftM (fromIntegral :: Word8 -> Int) B.get) B.get
      'P' -> liftM3 PatchPath (liftM (fromIntegral :: Word8 -> Int) B.get) (liftM (fromIntegral :: Word8 -> Int) B.get) B.get
      'i' -> liftM WithFid B.get
      'I' -> liftM ( WithKnownFid . SBS.pack ) B.get
      't' -> liftM WithTag B.get
      '_' -> liftM ( WithTag . flip ( M.findWithDefault "-" ) freqTags ) B.get
      'a' -> liftM2 WithProperty B.get B.get
      '^' -> liftM2 ( WithProperty . flip ( M.findWithDefault "-" ) freqProps ) B.get B.get
      '1' -> return Seen
      'm' -> return Mark
      'M' -> return SeenMark
      '0' -> return Missing
      x -> error $ "Wrong tag: (code " ++ show ( fromEnum x ) ++ ")"

==> file4life-1.2.3-source.tgz: file4life-1.2.3/src/F4L/V0/Op.hs <==
{- This file is part of file4life, a file backup program written in Haskell.
 - file4life is Copyright (C) Antonio Bellezza 2014-2017
 - file4life is free software: you can redistribute it and/or modify
 - it under the terms of the GNU General Public License as published by
 - the Free Software Foundation, either version 3 of the License, or
 - (at your option) any later version.

 - file4life is distributed in the hope that it will be useful,
 - but WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 - GNU General Public License for more details.

 - You should have received a copy of the GNU General Public License
 - along with file4life.  If not, see <http://www.gnu.org/licenses/>.
 -}
{-# LANGUAGE MultiParamTypeClasses, FlexibleContexts, RankNTypes #-}

module F4L.V0.Op (
  Op(..),
  Requirement(..)
  ) where

import F4L

import Control.DeepSeq
import Control.Monad.State.Strict
import qualified Data.Binary as B
import qualified Data.ByteString.Short as SBS
import Data.Binary( Binary )
import qualified Data.Map.Strict as M
import Data.Word( Word8 )


data Requirement = MergePriority !Priority
                 | OverridePriority !Priority
                 deriving (Eq, Show)


instance NFData Requirement where
  rnf ( MergePriority x ) = rnf x
  rnf ( OverridePriority x ) = rnf x


instance Binary Requirement where
  put ( MergePriority a ) = B.put a
  put ( OverridePriority a ) = B.put '!' >> B.put a
  -- TODO: avoid code duplication with above
  get = do
    t <- B.get
    case t of
      '!' -> liftM OverridePriority B.get
      '0' -> return $ MergePriority Skip
      'a' -> return $ MergePriority DefaultPriority
      'p' -> liftM ( MergePriority . Priority ) B.get
      'z' -> return $ MergePriority AsManyAsPossible
        

data Op = StartSession SessionVersion SessionId EnvironmentId TimeStamp Int
        | EndSession
        | AddEnvironmentLabel EnvironmentId String
        | AddStorageLabel StorageId String
        | WithStorage StorageId FilePath
        | WithRequire Requirement -- ^ set requirement for following files
        | MTag String -- ^ add tag for following files
        | RetailPath Int FilePath -- ^ remove n chars and append new part
        | PatchPath Int Int FilePath -- ^ cut from pos1 to pos2 and replace with new part
        | BackupPath -- ^ path within storage is standard backup path
        | WithFid FileId
        | WithKnownFid SBS.ShortByteString
        | WithTag String -- ^ add tag for next file or path
        | WithProperty String String -- ^ add property for next file or path
        | Seen -- ^ mark path as seen
        | Mark -- ^ add attributes
        | SeenMark -- ^ Seen + Mark
        | Missing -- ^ no file present in path
        deriving (Show, Eq)


instance NFData Op where
  rnf ( StartSession _ a b _ _ ) = rnf a `seq` rnf b
  rnf ( AddEnvironmentLabel a b ) = rnf a `seq` rnf b
  rnf ( AddStorageLabel a b ) = rnf a `seq` rnf b
  rnf ( WithStorage a b ) = rnf a `seq` rnf b
  rnf ( WithRequire a ) = rnf a
  rnf ( MTag a ) = rnf a
  rnf ( RetailPath a b ) = a `seq` rnf b
  rnf ( PatchPath a b c ) = a `seq` b `seq` rnf c
  rnf ( WithFid a ) = rnf a
  rnf ( WithKnownFid a ) = rnf a
  rnf ( WithTag a ) = rnf a
  rnf ( WithProperty a b ) = rnf a `seq` rnf b
  rnf x = x `seq` ()


{---------------------------------------- Binary ----------------------------------------}

swap (a, b) = (b, a)

freqTags = M.fromList $
           [ ( 'p', "picture" )
           , ( 'm', "email" )
           ]

freqTags' = M.fromList $ map swap $ M.toList freqTags

freqProps = M.fromList $
           [ ( 'D', "exif:DateTime" )
           ]
freqProps' = M.fromList $ map swap $ M.toList freqProps


instance Binary Op where
  put ( StartSession sver sessid env ts l ) = B.put 'S' >> B.put sver >> B.put sessid >> B.put env >> B.put ts >> B.put l
  put EndSession = B.put 'E'
  put ( AddEnvironmentLabel env tag ) = B.put 'L' >> B.put env >> B.put tag
  put ( AddStorageLabel env tag ) = B.put 'l' >> B.put env >> B.put tag
  put ( WithStorage sid path ) = B.put 's' >> B.put sid >> B.put path
  put ( WithRequire req ) = B.put 'r' >> B.put req
  put ( MTag s ) = B.put 'T' >> B.put s
  -- put ( MProperty k v ) = B.put 'P' >> B.put k >> B.put v
  put BackupPath = B.put 'b'
  put ( RetailPath n path ) = B.put 'p' >> B.put ( fromIntegral n :: Word8 ) >> B.put path
  put ( PatchPath d k path ) = B.put 'P' >> B.put ( fromIntegral d :: Word8 ) >> B.put ( fromIntegral k :: Word8 ) >> B.put path
  put ( WithFid fid ) = B.put 'i' >> B.put fid
  put ( WithKnownFid h ) = B.put 'I' >> B.put ( SBS.unpack h )
  put ( WithTag s ) = case M.lookup s freqTags' of
    Nothing ->
      B.put 't' >> B.put s
    Just x ->
      B.put '_' >> B.put x
  put ( WithProperty k v ) = case M.lookup k freqProps' of 
    Nothing ->
      B.put 'a' >> B.put k >> B.put v
    Just x ->
      B.put '^' >> B.put x >> B.put v
  put Seen = B.put '1'
  put Mark = B.put 'm'
  put SeenMark = B.put 'M'
  put Missing = B.put '0'
  -- put Nop = return ()

  get = do
    t <- B.get
    case t of
      'S' -> liftM5 StartSession B.get B.get B.get B.get B.get
      'E' -> return EndSession
      'L' -> liftM2 AddEnvironmentLabel B.get B.get
      'l' -> liftM2 AddStorageLabel B.get B.get
      's' -> liftM2 WithStorage B.get B.get
      'r' -> liftM WithRequire B.get
      'T' -> liftM MTag B.get
      'b' -> return BackupPath
      'p' -> liftM2 RetailPath (liftM (fromIntegral :: Word8 -> Int) B.get) B.get
      'P' -> liftM3 PatchPath (liftM (fromIntegral :: Word8 -> Int) B.get) (liftM (fromIntegral :: Word8 -> Int) B.get) B.get
      'i' -> liftM WithFid B.get
      'I' -> liftM ( WithKnownFid . SBS.pack ) B.get
      't' -> liftM WithTag B.get
      '_' -> liftM ( WithTag . flip ( M.findWithDefault "-" ) freqTags ) B.get
      'a' -> liftM2 WithProperty B.get B.get
      '^' -> liftM2 ( WithProperty . flip ( M.findWithDefault "-" ) freqProps ) B.get B.get
      '1' -> return Seen
      'm' -> return Mark
      'M' -> return SeenMark
      '0' -> return Missing
      x -> error $ "Wrong tag: (code " ++ show ( fromEnum x ) ++ ")"
